/*******************************************************************************
* ts.h: Buffer classes definitions                                             *
* (c)1999 VideoLAN                                                             *
*------------------------------------------------------------------------------*
*                                                                              *
*------------------------------------------------------------------------------*
* required headers:                                                            *
*  "common.h"                                                                  *
*  "ts.h"                                                                      *
*******************************************************************************/

#ifndef _BUFFER_H_
#define _BUFFER_H_


//------------------------------------------------------------------------------
// class C_Fifo
//------------------------------------------------------------------------------
// Implementation of a rotative fifo of TS packets
//------------------------------------------------------------------------------
class C_Fifo
{
 public:
  C_Fifo(unsigned int iBuffSize);
  ~C_Fifo();

  int Push(C_TsPacket* pPacket);
  C_TsPacket* Pop();

  inline int Stored()
  { return iStored; };
  inline int Size()
  { return iSize; };

  C_TsPacket* operator [] (unsigned int iIndex);

 protected:
  // Size of the buffer
  unsigned int iSize;
  // Number of TS_Packet stored in the Fifo
  unsigned int iStored;
  // Where to push packets in the buffer ?
  unsigned int iWhereToPush;
  // Where to pull them ?
  unsigned int iWhereToPop;
  // Buffer
  C_TsPacket** apBuff;
};



//------------------------------------------------------------------------------
// Class C_NetList
//------------------------------------------------------------------------------
// 
//------------------------------------------------------------------------------
class C_NetList
{
 public:
  C_NetList(unsigned int iSize = 2^8);
  ~C_NetList();
  
  C_TsPacket* GetPacket();
  void ReleasePacket(C_TsPacket* pPacket);

 protected:
  // The buffer of TS packets itself
  C_TsPacket* aPackets;
  // Array of pointers to the free packets (ie that never were poped) and
  // index of the first element of the table that points to a such packet
  C_TsPacket** apFreePackets;
  unsigned int iFirstFree;
  // Size of the 2 previous arrays
  unsigned int iBuffSize;
};





#else
#error "Multiple inclusions of buffer.h"
#endif
